import unittest
from deque import Deque

class TestDeque(unittest.TestCase):

	def test_create(self):
		"""Test creation of a deque."""
		dq = Deque()
		self.assertTrue(dq.empty())
	
	def test_push_back(self):
		"""Test push back"""
		dq = Deque()
		self.assertTrue(dq.empty())
		dq.push_back(1)
		self.assertFalse(dq.empty())

	def test_pop_back(self):
		"""Test pop back"""
		dq = Deque()
		self.assertRaises(Exception, dq.pop_back)
		dq.push_back(1)
		dq.pop_back()
		self.assertTrue(dq.empty())
		self.assertRaises(Exception, dq.pop_back)

	def test_back(self):
		"""Test back"""
		dq = Deque()
		self.assertRaises(Exception, dq.back)
		dq.push_back(1)
		self.assertEqual(dq.back(), 1)
		dq.push_back(2)
		self.assertEqual(dq.back(), 2)
		dq.push_back(3)
		self.assertEqual(dq.back(), 3)
		dq.push_back(4)
		self.assertEqual(dq.back(), 4)
		dq.pop_back()
		self.assertEqual(dq.back(), 3)
		dq.pop_back()
		self.assertEqual(dq.back(), 2)
		dq.pop_back()
		self.assertEqual(dq.back(), 1)
		dq.pop_back()
		self.assertRaises(Exception, dq.back)

	def test_push_front(self):
		"""Test push front"""
		dq = Deque()
		self.assertTrue(dq.empty())
		dq.push_front(1)
		self.assertFalse(dq.empty())

	def test_pop_front(self):
		"""Test pop front"""
		dq = Deque()
		self.assertRaises(Exception, dq.pop_front)
		dq.push_front(1)
		dq.pop_front()
		self.assertTrue(dq.empty())
		self.assertRaises(Exception, dq.pop_front)

	def test_front(self):
		"""Test front"""
		dq = Deque()
		self.assertRaises(Exception, dq.front)
		dq.push_front(1)
		self.assertEqual(dq.front(), 1)
		dq.push_front(2)
		self.assertEqual(dq.front(), 2)
		dq.push_front(3)
		self.assertEqual(dq.front(), 3)
		dq.push_front(4)
		self.assertEqual(dq.front(), 4)
		dq.pop_front()
		self.assertEqual(dq.front(), 3)
		dq.pop_front()
		self.assertEqual(dq.front(), 2)
		dq.pop_front()
		self.assertEqual(dq.front(), 1)
		dq.pop_front()
		self.assertRaises(Exception, dq.front)

	def test_interactions(self):
		"""Test interactions among members"""
		dq = Deque()
		dq.push_front("a")
		dq.push_back(1)
		self.assertEqual(dq.front(), "a")
		self.assertEqual(dq.back(), 1)
		dq.push_front("b")
		dq.push_back(2)
		self.assertEqual(dq.front(), "b")
		self.assertEqual(dq.back(), 2)
		dq.push_front("c")
		dq.push_back(3)
		self.assertEqual(dq.front(), "c")
		self.assertEqual(dq.back(), 3)
		dq.pop_front()
		dq.pop_front()
		dq.pop_front()
		self.assertEqual(dq.front(), 1)
		dq.push_front("hello")
		dq.push_front("hi")
		dq.push_front("howdy")
		dq.pop_back()
		dq.pop_back()
		dq.pop_back()
		self.assertEqual(dq.back(), "hello")
		
if __name__ == '__main__':
    unittest.main()
